#include"rpglib.h"

#define QUERYBUFLEN 1024

int librpg_debuglevel=0;
// default constructor -- initialize everything
PgConnection::PgConnection()
{
    pgConn=NULL;
    pgCloseConnection=1;
}

PgConnection::PgConnection(const char* conninfo)
{
    if(Connect(conninfo) == CONNECTION_BAD)
    {
	throw PgException(DBName(),PGRES_FATAL_ERROR,"%s",(const char *)PQerrorMessage(pgConn));
    }
}

ConnStatusType PgConnection::Connect(const char* conninfo)
{
    ConnStatusType cst;
    // Turn the trace on
    if(librpg_debuglevel>0)
    {
	FILE *debug = fopen("/tmp/trace.out","w");
	PQtrace(pgConn, debug);
    }
  
    // Connect to the database
    pgConn = PQconnectdb(conninfo);
  
    // Status will return either CONNECTION_OK or CONNECTION_BAD
    cst =  PQstatus(pgConn);
    if(CONNECTION_OK == cst) pgCloseConnection = (ConnStatusType)1;
    else pgCloseConnection = (ConnStatusType)0;

    return cst;
}

PgConnection::~PgConnection()
{
  // Terminate the debugging output if it was turned on
  if(librpg_debuglevel>0) PQuntrace(pgConn);
  
  // Close the conneciton only if needed
  // This feature will most probably be used by the derived classes that
  // need not close the connection after they are destructed.
  if ( pgCloseConnection ) {
       if(pgConn) PQfinish(pgConn);
  }
}


const char* PgConnection::DBName()
{ 
    return (const char *)PQdb(pgConn); 
}


void PgConnection::ExecCommand(const char* query,...) throw (PgException)
{
    va_list args;
    char anweisung[QUERYBUFLEN];

    va_start(args,query); 
    vsnprintf(anweisung,QUERYBUFLEN,query,args);
    va_end(args);

    PGresult *res = PQexec(pgConn, anweisung);
    
    if (!res || PQresultStatus(res) != PGRES_COMMAND_OK)
    {
	ExecStatusType err=PQresultStatus(res);
	const char *msg = PQerrorMessage(pgConn);
	PQclear(res);
	PQreset(pgConn);
	throw PgException(DBName(),err,"Command %s failed:%s",anweisung,msg);
    }
    PQclear(res);
}

PGresult* PgConnection::ExecRetTuples(const char* query,...) throw (PgException)
{
   va_list args;
   char anweisung[QUERYBUFLEN];

   va_start(args,query); 
   vsnprintf(anweisung,QUERYBUFLEN,query,args);
   va_end(args);

   PGresult *res = PQexec(pgConn, anweisung);
   if (!res || PQresultStatus(res) != PGRES_TUPLES_OK)
   {
       ExecStatusType err=PQresultStatus(res);
       const char *msg = PQerrorMessage(pgConn);
       PQclear(res);
       PQreset(pgConn);
       throw PgException(DBName(),err,"Command %s failed:%s",anweisung,msg);
   }
   return res;
}

void PgConnection::transaction(char *cmd) throw (PgException)
{
    PGresult *res=PQexec(pgConn,cmd);
    if (!res || PQresultStatus(res) != PGRES_COMMAND_OK)
    {
	ExecStatusType err=PQresultStatus(res);
	PQclear(res);
	PQreset(pgConn);
	throw PgException(DBName(),err,"TRANSACTION %s failed",cmd);
    }
    PQclear(res);
}




