#ifndef RPGLIB_H
#define RPGLIB_H
#include<string.h>
#include<stdio.h>
#include<stdarg.h>

#include <stdio.h>
#include <string>

extern "C" {
#include "config.h"
#include "libpq-fe.h"
}

#define MESSBUFLEN 2048
class PgException
{
  char errmsg[MESSBUFLEN]; 
  ConnStatusType istatus;
public:
  PgException(const char *db,ExecStatusType status,const char *format,...)
   {
     va_list args;
     va_start(args,format);
     int l=strlen(db);
     strcpy(errmsg,db);errmsg[l]=' ';
     vsnprintf(errmsg+l+1,MESSBUFLEN-l-1,format,args);
     va_end(args);
   }
  const char *what() { return errmsg; }
  ConnStatusType status() { return istatus; }
};

class PgConnection
{
 protected:
    PGconn* pgConn;	// Connection Structures
    int pgCloseConnection; // Flag indicating whether the connection should be closed or not
    
 public:
    PgConnection();
    PgConnection(const char* conninfo); //use reasonable & environment defaults
   ~PgConnection(); 		        //close connection and clean up
   
   const char* DBName();
   const char* user() { return PQuser(pgConn); }
   const char* pass() { return PQpass(pgConn); }
   const char* host() { return PQhost(pgConn); }
   const char* port() { return PQport(pgConn); }
   const char* tty()  { return PQtty(pgConn); }
   const char* options() { return PQoptions(pgConn); }
   int backendPID()   { return PQbackendPID(pgConn); }

   ConnStatusType Connect(const char* conninfo);
 
   //send a command and check if it's OK
   void ExecCommand(const char* query,...) throw (PgException); 
   // send a command and check if tuples are returned
   PGresult* ExecRetTuples(const char* query,...) throw (PgException); 

   void reset() { PQreset(pgConn); }
 protected:
   void transaction(char *cmd) throw (PgException);
   friend class PgTransaction;
};

class PgTransaction : public PgConnection
{
 public:
    enum Status { Active,Aborted,Committed };
 private:
  Status status;
public:
  PgTransaction(PgConnection &conn) throw (PgException)
      {
	  pgConn = conn.pgConn;
	  pgCloseConnection=0;
	  status=Active;
          
	  transaction("BEGIN");
      }
  ~PgTransaction() throw (PgException)
      {
	  if(status == Active) transaction("ROLLBACK");
      }
  void commit() throw (PgException)
      {
	  transaction("COMMIT");
	  status=Committed;
      }
  void rollback() throw (PgException)
      {
	  transaction("ROLLBACK");
	  status=Aborted;
      }
  Status getstatus() { return status; }
};

class PgResultHandle
{
  PGresult *res;
 public:
  PgResultHandle() { res=NULL;}
  PgResultHandle(PGresult *r) { res=r; }
  void operator=(PGresult *r) { if(res) PQclear(res); res=r; }
  PGresult *ptr() { return res; }
  ~PgResultHandle() { if(res) PQclear(res); }
};
#endif

